import style from './style.css?inline'

type SnackbarType = 'success' | 'error' | 'warning' | 'info'

interface UseSnackBar {
  message: string
  type: SnackbarType
}

class SnackBar extends HTMLElement {
  private _shadowRoot: ShadowRoot

  constructor() {
    super()

    this._shadowRoot = this.attachShadow({ mode: 'open' })
    this._shadowRoot.innerHTML = `
      <style>${style}</style>
      <div class="snackbar">
        <i class="mdi mdi-alert-outline"></i>
        <p></p>
      </div>
    `
  }

  static get observedAttributes() {
    return ['message', 'open', 'type']
  }

  attributeChangedCallback(name: string, _oldValue: string, newValue: string) {
    const snackbar = this._shadowRoot.querySelector<HTMLDivElement>('.snackbar')
    const p = this._shadowRoot.querySelector<HTMLParagraphElement>('.snackbar p')
    const i = this._shadowRoot.querySelector<HTMLElement>('.snackbar i')

    if (name === 'open') {
      if (newValue === 'true') {
        snackbar?.classList.add('open')
        setTimeout(() => snackbar?.classList.remove('open'), 3000)
      } else {
        snackbar?.classList.remove('open')
      }
    }

    if (name === 'message') {
      console.log(p)
      p!.innerText = newValue
    }

    if (name === 'type') {
      switch (newValue as SnackbarType) {
        case 'success':
          snackbar?.classList.add('success')
          i!.className = 'mdi mdi-check-circle-outline'
          break
        case 'error':
          snackbar?.classList.add('error')
          i!.className = 'mdi mdi-alert-circle-outline'
          break
        case 'warning':
          snackbar?.classList.add('warning')
          i!.className = 'mdi mdi-alert-outline'
          break
        case 'info':
          snackbar?.classList.add('info')
          i!.className = 'mdi mdi-information-outline'
          break
      }
    }
  }
}

customElements.define('snack-bar', SnackBar)

export const useSnackBar = ({ message, type }: UseSnackBar) => {
  const snackbar = document.querySelector<SnackBar>('snack-bar')
  console.log(snackbar)

  snackbar?.setAttribute('message', message)
  snackbar?.setAttribute('type', type)
  snackbar?.setAttribute('open', 'true')
}